@@ -14,7 +14,7 @@
 #ifndef PQXX_NONTRANSACTION_H
 #define PQXX_NONTRANSACTION_H
 
-#include "pqxx/connection.h"
+#include "pqxx/connectionitf.h"
 #include "pqxx/result.h"
 #include "pqxx/transactionitf.h"
 
@@ -27,7 +27,7 @@ namespace pqxx
 /// Simple "Transaction" class offering no transactional integrity.
 /**
  * NonTransaction, like Transaction or any other TransactionItf-derived class,
- * provides access to a database through a Connection.  Unlike its siblings,
+ * provides access to a database through a connection.  Unlike its siblings,
  * however, NonTransaction does not maintain any kind of transactional 
  * integrity.  This may be useful eg. for read-only access to the database that
  * does not require a consistent, atomic view on its data.
@@ -43,11 +43,11 @@ class PQXX_LIBEXPORT NonTransaction : public TransactionItf
 public:
   /// Constructor.
   /** Create a "dummy" transaction.
-   * @param C the Connection that this "transaction" will operate on.
+   * @param C the connection that this "transaction" will operate on.
    * @param NName an optional name for the transaction, beginning with a letter
    * and containing only letters and digits.
    */
-  explicit NonTransaction(Connection &C, 
+  explicit NonTransaction(ConnectionItf &C, 
 		          const PGSTD::string &NName=PGSTD::string()) :	//[t14]
     TransactionItf(C, NName) { Begin(); }
 
