@@ -57,7 +57,7 @@ class TestTrig : public Trigger
   bool m_Done;
 
 public:
-  explicit TestTrig(Connection &C) : Trigger(C, "trig"), m_Done(false) {}
+  explicit TestTrig(ConnectionItf &C) : Trigger(C, "trig"), m_Done(false) {}
 
   virtual void operator()(int be_pid)
   {
