@@ -93,7 +93,7 @@ int main(int argc, char *argv[])
 
     // Set up two connections to the backend: one to read our original table,
     // and another to write our copy
-    Connection C(argv[1], false);
+    LazyConnection C(argv[1]);
 
     // Select our original and destination table names
     string TableName = "testtable";
